# perl5 -w

# this program rips the sprites from a set of wads from a directory, dumps them
# for the IGJ0 code to use, and creates a web page showing the animations

# notes for web page:
# - usually a-d are walk, e-g are attack, h
# - @todo how do you size the guys correctly?  can't really take bitmap size...what about encoding in the filename?
# - separate dudes.html and objects.html...objects are things that aren't sets of animations with direction?

use File::DosGlob 'glob';
use File::Copy;
my %sprites = ();

my $rootdir = "sprites";
my $waddir = "wads";
my @wads = glob "$waddir/*.wad";
# @wads = ("wads/hexen.wad");

if(not -e $rootdir) {
  mkdir $rootdir;
}
foreach (@wads) {
  $_ = lc $_;
  /$waddir\/(.*)\.wad/i;
  my $wad = $1;
  my $deudir = "$rootdir/$wad";
  my $spritedir = "$deudir/sprites";
  if((not -e $deudir) || (not -e "$deudir/wadinfo.txt") || (-M "$deudir/wadinfo.txt" > -M $_)) {
    print "Ripping $wad...\n";
    mkdir $deudir;
    system "deutex -doom $waddir -overwrite -sprites -dir $deudir -bmp -extract $_";
    if(-e $spritedir) {
      #
      # custom per-wad fixups!
      #
      if($wad eq "hexen" || $wad eq "heretic") {
        print "Fix: deleting $wad 'arti' incompatible sprites...\n";
        unlink glob "$spritedir/arti*.bmp";
      } elsif($wad eq "gold64") {
        print "Fix: deleting $wad incompatible sprites...\n";
        unlink "$spritedir/bds1a4a6.bmp";
      } elsif($wad eq "atc2sf") {
        print "Fix: fixing $wad incompatible sprites...\n";
        # mis-named rot=0 sprites
        rename "$spritedir/bosse1.bmp", "$spritedir/bosse0.bmp";
        rename "$spritedir/bossf1.bmp", "$spritedir/bossf0.bmp";
        rename "$spritedir/bossg1.bmp", "$spritedir/bossg0.bmp";
        # there is no d frame on spid
        rename "$spritedir/spida1d1.bmp", "$spritedir/spida1.bmp";
        rename "$spritedir/spida5d5.bmp", "$spritedir/spida5.bmp";
        # missing boss c 1 frame
        copy("$spritedir/bossd1.bmp", "$spritedir/bossc1.bmp");
      }
    }
  } else {
    print "Not ripping $wad because of wadinfo.txt date...\n";
  }

  (-e $spritedir) || printf "Warning: Deutex did not make $spritedir! No sprites in wad?";
}

my @spritedirs = glob "$rootdir/*";
foreach (@spritedirs) {
  /$rootdir\/(.*)/;
  my $wad = lc $1;
  my $spritedir = "$_/sprites";
  my @sprite_files = glob "$spritedir/*.bmp";

  foreach (@sprite_files) {
    $_ = lc $_;
    if(/$spritedir\/(....)(.)([0-9])(.*)\.bmp/i) {
      my $name = $1;
      my $anim = $2;
      my $rot = $3;
      my $mirror = $4;

      $sprites{$wad}{$name}{$anim}{$rot} = [$_,0];      # [file, mirror?]
      if ($mirror) {
        if($mirror =~ /(.)([0-9])/) {
          $sprites{$wad}{$name}{$1}{$2} = [$_,1];       # [file, mirror?]
        } else {
          print "ignoring ill-formed sprite mirror $_\n";
        }
      }
    } else {
      print "ignoring ill-formed sprite $_\n";
    }
  }
}

open OUTPUT, ">dudes.html" || die;
print OUTPUT <<HERE;
<html><head><title>Dudes</title></head>
<body>
HERE

foreach (sort (keys %sprites)) {
  my $wad = $_;
  print "Generating gifs for $wad...\n";
  foreach (sort (keys %{$sprites{$wad}})) {
    my $name = $_;
    my $gif = "$rootdir/$wad/$name.gif";
    my @sys = ("util_combine",$gif);
    foreach (sort (keys %{$sprites{$wad}{$name}})) {
      my $anim = $_;
      my @rots = ();

      if(exists $sprites{$wad}{$name}{$anim}{0}) {
        # it's a rot=0 lump, as they say
        @rots = (0);
      } else {
        # it's got all eight rotations, so display the front and side
        @rots = (1,3);
      }
      my @systmp = ();
      my $success = 1;
      foreach (@rots) {
        if(exists $sprites{$wad}{$name}{$anim}{$_}) {
          my ($f,$m) = @{$sprites{$wad}{$name}{$anim}{$_}};
          my $m = ($m == 1 ? "m" : "");
          my $s = uc($anim);
          if ($_ == 0) {
            $s .= " 1";
          } elsif ($_ == 1) {
            $s .= " 8$m";
          } elsif ($_ == 3) {
            $s = " $m"
          }
          push @systmp, $f;
          push @systmp, "\"$s\"";
        } else {
          $success = 0;
        }
      }
      if($success) {
        push @sys, @systmp;
      } else {
        print "ignoring $wad $name $anim (@rots)\n";
      }
    }
    if(@sys > 2) {
      print OUTPUT <<HERE;
<table border=0><tr><td>$rootdir/$wad/sprites/$name</td><td><img src=\"$gif\"></td></tr></table><br>
HERE
      system @sys;
    }
  }
}

print OUTPUT <<HERE;
</body>
HERE
